///-------------------------------------------------------------------------------------------------
/// 
/// \file ovpCBoxAlgorithmTactiloController.cpp
/// \brief Functions of the Box Tactilo Controller.
/// \author Tobias Baumann (TH-Nürnberg).
/// \version 1.0.
/// \date Mon Feb 21 14:59:56 2022.
/// \copyright <a href="https://choosealicense.com/licenses/agpl-3.0/">GNU Affero General Public License v3.0</a>.
/// 
///-------------------------------------------------------------------------------------------------

//includes
#include "ovpCBoxAlgorithmTactiloController.h"
#include <string>

using namespace OpenViBE;
using namespace /*OpenViBE::*/Kernel;
using namespace /*OpenViBE::*/Plugins;
using namespace /*OpenViBE::Plugins::*/Tactilebci;

/*******************************************************************************/

bool CBoxAlgorithmTactiloController::initialize()
{
	m_StimDecoder.initialize(*this, 0);
	m_StimEncoder.initialize(*this, 0);
	
	//get Box Settings
	m_PortName         = FSettingValueAutoCast(*this->getBoxAlgorithmContext(), 0);
	m_RowBase          = FSettingValueAutoCast(*this->getBoxAlgorithmContext(), 1);
	m_NumberofTactilos = 6; //if this value is specified via box settings this line is not needed
	//NumberofTactilos = FSettingValueAutoCast(*this->getBoxAlgorithmContext(), 3); //used if this value is set in box settings
	
	//open serial port and set baudrate to 115200
	m_Port.open(m_PortName.toASCIIString());
  	m_Port.set_option(boost::asio::serial_port::baud_rate(115200));
	
	if(m_Port.is_open())
	{
		this->getLogManager() << LogLevel_Info << "Connected to Serial Port: " << m_PortName << "\n";
		m_Port.write_some(boost::asio::buffer("b"));
	}
	
	return true;
}

/*******************************************************************************/

bool CBoxAlgorithmTactiloController::uninitialize()
{
	this->getLogManager() << LogLevel_Info << "Disconnect from Serial Port: " << m_PortName << "\n";
	m_Port.write_some(boost::asio::buffer("e"));
	m_Port.close();

	m_StimDecoder.uninitialize();
	m_StimEncoder.uninitialize();
	
	return true;
}

/*******************************************************************************/

bool CBoxAlgorithmTactiloController::processInput(const size_t index)
{
	// some pre-processing code if needed...

	// ready to process !
	getBoxAlgorithmContext()->markAlgorithmAsReadyToProcess();

	return true;
}

/*******************************************************************************/
bool CBoxAlgorithmTactiloController::process()
{
	
	// the static box context describes the box inputs, outputs, settings structures
	const IBox& staticBoxContext = this->getStaticBoxContext();
	// the dynamic box context describes the current state of the box inputs and outputs (i.e. the chunks)
	IBoxIO& boxContext       = this->getDynamicBoxContext();
	
	uint64_t StimulationID   = 0;
	uint64_t ChunkStartTime  = 0;
	uint64_t ChunkEndTime    = 0;
	uint64_t Size            = 0;
	const uint8_t*  Buffer   = nullptr;    

	//iterate over all chunk on input 0
	for (uint64_t i = 0; i < boxContext.getInputChunkCount(0); ++i)
	{
		// decode the chunk i
		m_StimDecoder.decode(i);

		if(m_StimDecoder.isBufferReceived())
		{
			//check received stimulations
			IStimulationSet* StimSet = m_StimDecoder.getOutputStimulationSet();
			for(uint64_t j=0; j<StimSet->getStimulationCount(); j++)
			{
				StimulationID   = StimSet->getStimulationIdentifier(j);
				
				if(StimulationID >= m_RowBase && StimulationID < (m_RowBase + m_NumberofTactilos))
				{				
					m_TactiloNr = StimulationID-m_RowBase+1;
				}
				if(StimulationID == OVTK_StimulationId_VisualStimulationStart)
				{
					this->getLogManager() << LogLevel_Info << "Tactilo Nr. " << m_TactiloNr << " ON\n";
					m_Port.write_some(boost::asio::buffer(std::to_string(m_TactiloNr)));
				}
				if(StimulationID == OVTK_StimulationId_VisualStimulationStop)
				{
					this->getLogManager() << LogLevel_Info << "Tactilo Nr. " << m_TactiloNr << " OFF\n";
					m_Port.write_some(boost::asio::buffer(std::to_string(m_TactiloNr)));
				}
			}
		}
		// forward input chunks
		boxContext.getInputChunk(0, i, ChunkStartTime, ChunkEndTime, Size, Buffer);
		boxContext.appendOutputChunkData(0, Buffer, Size);
		boxContext.markOutputAsReadyToSend(0, ChunkStartTime, ChunkEndTime);
		boxContext.markInputAsDeprecated(0, i);
	}

	return true;
}

/*******************************************************************************/